#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-template-field
------------

Tests for `django-template-field` fields.
"""
import pytest
from django.test.utils import override_settings

from .models import RelatedToTemplatedText, TemplatedText


pytestmark = pytest.mark.django_db


class TestTemplatefield(object):

    def setUp(self):
        self.tmpl = "{{ template_var }} are pretty neat"
        self.model = TemplatedText(value=self.tmpl)
        self.model.save()
        self.related_model = RelatedToTemplatedText(templated_text=self.model)
        self.related_model.save()

    def test_none(self):
        tt = TemplatedText(value=None)
        tt.save()
        assert TemplatedText.objects_unrendered.first().value == None

    def test_unrendered(self):
        self.setUp()
        tt = TemplatedText.objects_unrendered.first()
        assert tt.value == self.tmpl

    def test_rendered(self):
        self.setUp()
        tt = TemplatedText.objects_rendered.first()
        assert tt.value == "Dogs are pretty neat"

    def test_rendered_related(self):
        self.setUp()
        rttt = RelatedToTemplatedText.objects.first()
        assert rttt.templated_text.value == "Dogs are pretty neat"

    def test_qs_context(self):
        self.setUp()
        tt = TemplatedText.objects_rendered.with_context(
            {'template_var': 'Cats'}).first()
        assert tt.value == "Cats are pretty neat"



from django.db import models


class RenderTemplateManager(models.Manager):
    use_for_related_fields = True

    def get_queryset(self):
        qs = super(RenderTemplateManager, self).get_queryset()
        qs.query.add_context('show_rendered', True)
        return qs

    def with_context(self, extra_context):
        qs = self.get_queryset()
        qs.query.add_context('tmpl_context', extra_context)
        return qs



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_setting(setting):
    """ Get the specified django setting, or it's default value """
    defaults = {
        # The context to use for rendering fields
        'TEMPLATE_FIELD_CONTEXT': {},
        # When this is False, don't do any TemplateField rendering
        'TEMPLATE_FIELD_RENDER': True
    }
    try:
        return getattr(settings, setting, defaults[setting])
    except KeyError:
        msg = "{0} is not specified in your settings".format(setting)
        raise ImproperlyConfigured(msg)



import os

INSTALLED_APPS = [
    'templatefield.test',
]

SECRET_KEY = 'secret'

HERE = os.path.dirname(os.path.abspath(__file__))
DB = os.path.join(HERE, 'testdb.sqlite')

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': DB,
        'TEST': {
            'NAME': DB,
        },
    },
}

MIDDLEWARE_CLASSES = ()

TEMPLATE_FIELD_CONTEXT = {'template_var': 'Dogs'}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_settings
-------------

Tests for `django-template-field` settings
"""
import json
import pytest
import sys

from django.core.exceptions import ImproperlyConfigured
from django.core.management import call_command
from django.test.utils import override_settings

from ..settings import get_setting
from .models import TemplatedText


pytestmark = pytest.mark.django_db


class TestSettings(object):
    def setUp(self):
        self.tmpl = "{{ template_var }} are pretty neat"
        self.model = TemplatedText(value=self.tmpl)
        self.model.save()

    @override_settings(TEMPLATE_FIELD_CONTEXT={'test_var': 'val'})
    def test_get_setting(self):
        assert get_setting('TEMPLATE_FIELD_CONTEXT') == {'test_var': 'val'}
        assert get_setting('TEMPLATE_FIELD_RENDER') == True
        pytest.raises(ImproperlyConfigured, get_setting, 'FAKE_SETTING')

    def test_render_true(self, capsys):
        self.setUp()
        call_command('dumpdata', 'test.templatedtext')
        out, _ = capsys.readouterr()
        fixture = json.loads(out)

        assert len(fixture) == 1
        assert fixture[0]['fields']['value'] == 'Dogs are pretty neat'

    def test_render_false(self, capsys):
        with override_settings(TEMPLATE_FIELD_RENDER=False):
            self.setUp()
            call_command('dumpdata', 'test.templatedtext')
            out, _ = capsys.readouterr()
            fixture = json.loads(out)
            fixture_value = fixture[0]['fields']['value']

            assert len(fixture) == 1
            assert fixture_value == '{{ template_var }} are pretty neat'



from django.db import models

from templatefield import fields, managers


class TemplatedText(models.Model):
    value = fields.TemplateTextField(null=True)

    objects_rendered = managers.RenderTemplateManager()
    objects_unrendered = models.Manager()


class RelatedToTemplatedText(models.Model):
    templated_text = models.ForeignKey(TemplatedText)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_admin
------------

Tests for `django-template-field` admin classes.
"""
import pytest

from django.contrib.admin.sites import AdminSite
from mock import MagicMock

from .models import RelatedToTemplatedText, TemplatedText
from ..admin import UnrenderedAdmin


class TestUnrenderedAdmin(object):
    def test_get_queryset(self):
        # Check the resulting context in both models
        for model in [RelatedToTemplatedText, TemplatedText]:
            admin = UnrenderedAdmin(model, AdminSite())
            qs = admin.get_queryset(MagicMock())
            assert qs.query.context == {}



from django.contrib import admin


class UnrenderedAdmin(admin.ModelAdmin):
    """ Make sure template fields aren't rendered in the Django admin. """

    def get_queryset(self, request):
        """ Remove ``show_rendered`` from the context, if it's there. """
        qs = super(UnrenderedAdmin, self).get_queryset(request)
        if 'show_rendered' in qs.query.context:
            del qs.query.context['show_rendered']
        return qs



from django.db import models
from django.template import Context, Template
from .settings import get_setting


class TemplateTextField(models.TextField):

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return None

        should_render = get_setting('TEMPLATE_FIELD_RENDER')
        if not context.get('show_rendered') or not should_render:
            # `show_rendered` flag is unset, return the templeate
            return value

        # Return the rendered template
        template = Template(value)
        context_dict = {}
        context_dict.update(get_setting('TEMPLATE_FIELD_CONTEXT'))
        context_dict.update(context.get('tmpl_context', {}))
        return template.render(Context(context_dict))
